<?php
/**
 * Holds all database access classes and functions
 *
 * @author David Bisset <dbisset@dimensionmedia.com>
 * @package Community Activate for BuddyPress
 * @subpackage classes
 * @version 0.1
 *
 */

class BPCA_Invite {
	
	/**
	 * Invite ID
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite()
	 * @since 0.1
	 */
	var $id;

	/**
	 * Activated User's ID
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite()
	 * @since 0.1
	 */
	var $activated_user_id;

	/**
	 * Invite Initiator User's ID
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite()
	 * @since 0.1
	 */
	var $initiator_user_id;

	/**
	 * POTENTIAL (meaning maybe) Activated User's ID (Temp, Just For Readability)
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite()
	 * @since 0.1
	 */
	var $potential_activated_user_id;

	/**
	 * Constructor
	 *
	 * @access public
	 * @global object $bp BuddyPress global settings
	 * @param string|array $args See populate()
	 * @see BPCA_Invite::populate()
	 * @since 0.1
	 * @todo Change bools to strings to better describe what variable is used for.
	 */
	function bpcaa_invite( $args='' ) {
		global $bp;

		$defaults = array(
			'id' => 0,
			'slug' => '',
			'populate_extras' => true,
			'user_id' => 0
		);

		$r = wp_parse_args( $args, $defaults );
		extract( $r, EXTR_SKIP );

//		if ( !$user_id )
//			$user_id = $bp->loggedin_user->id;

		if ( $id ) {
			$this->populate( $id, 0 );
		} else if ($user_id) { 
			$this->populate( 0, $user_id );	// currently not coded really
		}
	}



	/**
	 * Get the Invite from the database
	 *
	 * @param string|array $args See BPCA_Invite_Template::BPCA_Invite_Template() inline documentation for explanation of this argument
	 * @access public
	 * @static
	 * @global object $bp BuddyPress global settings
	 * @global wpdb $wpdb WordPress database object
	 * @since 0.1
	 * @see BPCA_Invite_Template::BPCA_Invite_Template
	 * @return array The Invites found matching the criteria (and the total number of, for pagination)
	 */
	function get( $args ) {
		global $bp, $wpdb;

		$defaults = array(
			'skip_detail_page_result' => true,
			'limit' => 0,  // limit
			'page' => 1,  // page 1 without a per_page will result in no pagination.
			'per_page' => 20, // results per page
			'populate_extras' => true,
			'search_terms' => '',
			'id' => '', // only type=single
			'slug' => '',  // only type=single
			'type' => 'all',  // all | active | inactive | unlocked | locked | single (and for member profile page - active_by_action | alphabetical | eventcount | points | newest)
			'user_id' => 0,  // for type=unlocked and populate_extras
			'action' => '' // for type=active_by_action and user_id and populate_extras, and type=unlocked with all of the "order by" filters
		);

		$r = wp_parse_args( $args, $defaults );
		extract( $r, EXTR_SKIP );

		switch ( $type ) {
			case 'single':
				$sql = $wpdb->prepare( "SELECT wp_usermeta.user_id, wp_users.user_login, wp_users.user_email, wp_users.display_name, wp_users.user_registered FROM wp_usermeta JOIN wp_users ON wp_users.ID = wp_usermeta.user_id " );
			break; 

			default:
			case 'all':
				$sql = $wpdb->prepare( "SELECT wp_usermeta.user_id, wp_users.user_login, wp_users.user_email, wp_users.display_name, wp_users.user_registered FROM wp_usermeta JOIN wp_users ON wp_users.ID = wp_usermeta.user_id " );
				$sql_total_count = $wpdb->prepare( "SELECT COUNT(user_id) FROM wp_usermeta JOIN wp_users ON wp_users.ID = wp_usermeta.user_id " );
			break;
		}

		$sql = apply_filters( 'bpcaa_invite_get_type_sql', $sql, $limit, $page, $per_page, $populate_extras, $search_terms, $id, $slug, $type, $user_id, $action );
		$sql_total_count = apply_filters( 'bpcaa_invite_get_type_count_sql', $sql_total_count, $limit, $page, $per_page, $populate_extras, $search_terms, $id, $slug, $type, $user_id, $action );

		// Filters
		if ( in_array( $type, array( 'all', 'newest', 'alphabetical' ) ) ) {
			$temp_sql .= " WHERE meta_key='bpca_activated_by_userID' AND meta_value = '0'";
			$sql .= $temp_sql;
			$sql_total_count .= $temp_sql;
		} else if ( in_array( $type, array( 'single' ) ) ) {
			$temp_sql .= " WHERE meta_key='bpca_activated_by_userID' AND meta_value = '0' AND user_id = ".$user_id;
			$sql .= $temp_sql;
			$sql_total_count = 1;		
		}
				

		if ( $search_terms ) {
			$search_terms = like_escape( $wpdb->escape( $search_terms ) );

			$search_sql = " AND";

			$search_sql .= " ( wp_users.display_name LIKE '%%{$search_terms}%%' OR wp_users.user_login LIKE '%%{$search_terms}%%' )";
			$sql .= $search_sql;
			$sql_total_count .= $search_sql;
		} 

		if ( 'newest' == $type || 'newest' == $action ) {
			$sql .= " GROUP BY wp_users.ID ORDER BY wp_users.ID DESC ";
		} elseif ( 'alphabetical' == $type || 'alphabetical' == $action ) {
			$sql .= " GROUP BY wp_users.ID ORDER BY wp_users.display_name ASC";
		}
		
		$results = $wpdb->get_results( $sql );

		if ( 'single' == $type )
			$total_count = 1;
		else
			$total_count = $wpdb->get_var( $sql_total_count );
			
		
		return array( 'total' => $total_count, 'invites' => $results );
	}
	


	/**
	 * Activate A User Who Already is Waiting
	 *
	 */
	function activate_user(  ) {
		global $bp, $wpdb;

		$defaults = array(
			'initiator_user_id' => $this->initiator_user_id,
			'potential_activated_user_id' => $this->potential_activated_user_id,
			'date_activated' => $this->date_activated
		);

		$r = wp_parse_args( $args, $defaults );
		extract( $r, EXTR_SKIP );

                // get the current number of invites already done by the initiator user
                $current_number = intval(get_user_meta($initiator_user_id, 'bpca_invites_used', true));
                
                if (!is_int($current_number)) {
                    // If this isn't a number, then it's probably found which means this user has made his first invite
                    echo "test1"; exit;
                    $current_number = 0;
                    add_user_meta($initiator_user_id, 'bpca_invites_used', $current_number); // actual update right here
                }
                

		if ($potential_activated_user_id > 0 && $current_number < CACTIVATE_INVITE_LIMIT) {
			$result_first = $wpdb->query( $wpdb->prepare("UPDATE wp_users SET user_status = 0 WHERE ID = ".$potential_activated_user_id) );
			$result_second = $wpdb->query( $wpdb->prepare("UPDATE wp_usermeta SET meta_value = '".$initiator_user_id."' WHERE user_id = ".$potential_activated_user_id." AND meta_key = 'bpca_activated_by_userID'") );
			if ($result_second && $result_first) { 
				// let's add the activation date
				update_user_meta($potential_activated_user_id, 'bpca_activation_date', $date_activated);
				bpca_record_activity($potential_activated_user_id, '', $potential_activated_user_id, 'activate_user');
				bpca_record_activity($potential_activated_user_id, '', $potential_activated_user_id, 'got_activated');
                                // let's add 1 to the invites used for the initiator user
                                $current_number++;
                                update_user_meta($initiator_user_id, 'bpca_invites_used', $current_number); // actual update right here

				return true;
			} else {
				return false;
			}
		} else if ($current_number >= CACTIVATE_INVITE_LIMIT) {
			return "overlimit";
		} else {
                        return false;
                }
	}
	
	/**
	 * Get Activation Information For A User That Is Already Activated
	 *
	 */
	function get_activation_information(  ) {
		global $bp, $wpdb;
	
		$defaults = array(
			'activated_user_id' => $this->activated_user_id
		);

		$r = wp_parse_args( $args, $defaults );
		extract( $r, EXTR_SKIP );
		
		if ($activated_user_id > 0) {
			$sql = $wpdb->prepare( "SELECT wp_usermeta.meta_value, wp_users.user_login, wp_users.user_email, wp_users.display_name FROM wp_usermeta JOIN wp_users ON wp_users.ID = wp_usermeta.meta_value WHERE meta_key = 'bpca_activated_by_userID' AND user_id = ".$activated_user_id );
			$user_who_activated = $wpdb->get_results( $sql );
			$sql = $wpdb->prepare( "SELECT wp_usermeta.meta_value FROM wp_usermeta WHERE meta_key = 'bpca_activation_date' AND user_id = ".$activated_user_id );
			$activation_date = $wpdb->get_results( $sql );
			$sql = $wpdb->prepare( "SELECT wp_usermeta.meta_value FROM wp_usermeta WHERE meta_key = 'bpca_invites_used' AND user_id = ".$activated_user_id );
			$invites_used = $wpdb->get_results( $sql );
			return array( 'user_who_activated' => $user_who_activated, 'activation_date' => $activation_date, 'invites_used' => $invites_used );
		} else {
			return false;
		}
		
	}

	/**
	 * Get Invite Information (Like # of invites used) from any particular user
	 *
	 */
	function get_invite_information() {
		global $bp, $wpdb;

		$defaults = array(
			'user_id' => $this->user_id
		);

		$r = wp_parse_args( $args, $defaults );
		extract( $r, EXTR_SKIP );

		if ($user_id > 0) {
			$sql = $wpdb->prepare( "SELECT wp_usermeta.meta_value FROM wp_usermeta WHERE meta_key = 'bpca_invites_used' AND user_id = ".$user_id );
			$invites_used = $wpdb->get_results( $sql );
			return array( 'invites_used' => $invites_used );
		} else {
			return false;
		}

	}
	
}

/**
 * Get count of how many (active) Invites there are.
 *
 * @global object $bp BuddyPress global settings
 * @global wpdb $wpdb WordPress database object
 * @since 0.1
 */
function bpca_get_total_invites_count() {
	global $bp, $wpdb;

	if ( !$count = wp_cache_get( 'bpcaa_get_total_invites_count', 'bpcaa' ) ) {
		$temp_sql = '';

		$temp_sql = $wpdb->prepare( "WHERE meta_key='bpca_activated_by_userID' AND meta_value = '0'" );

		$count = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(user_id) FROM wp_usermeta {$temp_sql}" ) );
		wp_cache_set( 'bpcaa_get_total_invites_count', $count, 'bpcaa' );
	}

	return $count;
}

?>